﻿using HarmonyLib;
using System.Collections;
using System.Reflection;
using UnityEngine;
using UnityEngine.Networking;
using System.Collections.Generic;
using System;

public class DragonSharks : Mod
{
    private AssetBundle asset;
    Harmony harmonyInstance;
    string harmonyId = "com.aidanamite.DragonSharks";
    public static Material greenColor;
    public static Material redColor;
    public static Material blueColor;
    public static Material purpleColor;
    public static GameObject dragonPrefab;

    public IEnumerator Start()
    {
        harmonyInstance = new Harmony(harmonyId);
        harmonyInstance.PatchAll();
        var request = AssetBundle.LoadFromMemoryAsync(GetEmbeddedFileBytes("dragons.assets"));
        yield return request;
        asset = request.assetBundle;
        greenColor = asset.LoadAsset<Material>("GreenHP");
        redColor = asset.LoadAsset<Material>("RedHP");
        blueColor = asset.LoadAsset<Material>("BlueHP");
        purpleColor = asset.LoadAsset<Material>("PurpleHP");
        dragonPrefab = asset.LoadAsset<GameObject>("DragonPrefab");
        ToggleDisplay(true);
        Log("has been successfully loaded!");
    }

    public static List<SkinnedMeshRenderer> GetSharkSkins()
    {
        List<SkinnedMeshRenderer> result = new List<SkinnedMeshRenderer>();
        foreach (SkinnedMeshRenderer s in FindObjectsOfType<SkinnedMeshRenderer>())
            if (s.name.Contains("Shark"))
                result.Add(s);
        return result;
    }

    public static void ToggleDisplay(bool val)
    {
        if (val)
        {
            foreach (SkinnedMeshRenderer s in GetSharkSkins())
            {
                s.enabled = false;
                Transform parent = s.transform.parent.parent;
                if (!parent.Find("DRAGONSHARKS_OBJECT"))
                {
                    GameObject dragon = Instantiate(dragonPrefab, parent.position, parent.rotation, parent);
                    dragon.name = "DRAGONSHARKS_OBJECT";
                    int color = UnityEngine.Random.Range(1, 4);
                    if (color == 1)
                    {
                        dragon.GetComponentInChildren<SkinnedMeshRenderer>().sharedMaterial = greenColor;
                    }
                    else if (color == 2)
                    {
                        dragon.GetComponentInChildren<SkinnedMeshRenderer>().sharedMaterial = redColor;
                    }
                    else if (color == 3)
                    {
                        dragon.GetComponentInChildren<SkinnedMeshRenderer>().sharedMaterial = blueColor;
                    }
                    else
                    {
                        dragon.GetComponentInChildren<SkinnedMeshRenderer>().sharedMaterial = purpleColor;
                    }
                    dragon.transform.localScale = new Vector3(0.5f, 0.5f, 0.5f);
                    dragon.transform.localPosition = new Vector3(0, -3, 0);
                }
            }
        }
        else
        {
            int i = 0;
            foreach (SkinnedMeshRenderer s in GetSharkSkins())
            {
                s.enabled = true;
                Transform dragon = s.transform.parent.parent.Find("DRAGONSHARKS_OBJECT");
                if (dragon)
                {
                    DestroyImmediate(dragon.gameObject);
                    i++;
                }
            }
            Debug.Log("Successfully unloaded " + i + " dragons! :P");
        }

    }

    public void OnModUnload()
    {
        ToggleDisplay(false);
        harmonyInstance.UnpatchAll(harmonyId);
        asset.Unload(true);
        Log("has unloaded!");
    }
}

[HarmonyPatch(typeof(Network_Host_Entities), "CreateAINetworkBehaviour", new Type[] { typeof(AI_NetworkBehaviourType), typeof(Vector3), typeof(float), typeof(uint), typeof(uint), typeof(LandmarkEntitySpawner) })]
public static class HarmonyPatch_SpawnEntity
{
    private static void Postfix(AI_NetworkBehaviourType behaviourType)
    {
        if (behaviourType == AI_NetworkBehaviourType.Shark)
            DragonSharks.ToggleDisplay(true);
    }
}